@@ -23,18 +23,31 @@
    |          Hartmut Holzgraefe  <hholzgra@php.net>                      |
    |          Jani Taskinen       <sniper@iki.fi>                         |
    |          Daniel R. Kalowsky  <kalowsky@php.net>                      |
-   | PHP 4.0 updates:  Zeev Suraski <zeev@zend.com>                       |
+   | PHP 4.0 updates:  Zeev Suraski <zeev@php.net>                        |
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
-
 #ifndef PHP_IMAP_H
 #define PHP_IMAP_H
 
 #if HAVE_IMAP
 
 #if defined(HAVE_IMAP2000) || defined(HAVE_IMAP2001)
+
+ /* For now these appear on Windows, remove this check if it appears outside */
+# ifdef PHP_WIN32
+ /* Undefine these LOG defines to avoid warnings */
+#  undef LOG_EMERG
+#  undef LOG_CRIT
+#  undef LOG_ERR
+#  undef LOG_WARNING
+#  undef LOG_NOTICE
+#  undef LOG_DEBUG
+
+ /* c-client also redefines its own ftruncate */
+#  undef ftruncate
+# endif
+
  /* these are used for quota support */
 # include "c-client.h"	/* includes mail.h and rfc822.h */
 # include "imap4r1.h"	/* location of c-client quota functions */
