@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2016 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,8 +16,6 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
-
 #ifdef HAVE_CONFIG_H
 	#include "config.h"
 #endif
@@ -28,22 +26,22 @@
 #include "php_spl.h"
 
 /* {{{ spl_register_interface */
-void spl_register_interface(zend_class_entry ** ppce, char * class_name, const zend_function_entry * functions TSRMLS_DC)
+void spl_register_interface(zend_class_entry ** ppce, char * class_name, const zend_function_entry * functions)
 {
 	zend_class_entry ce;
-	
+
 	INIT_CLASS_ENTRY_EX(ce, class_name, strlen(class_name), functions);
-	*ppce = zend_register_internal_interface(&ce TSRMLS_CC);
+	*ppce = zend_register_internal_interface(&ce);
 }
 /* }}} */
 
 /* {{{ spl_register_std_class */
-PHPAPI void spl_register_std_class(zend_class_entry ** ppce, char * class_name, void * obj_ctor, const zend_function_entry * function_list TSRMLS_DC)
+PHPAPI void spl_register_std_class(zend_class_entry ** ppce, char * class_name, void * obj_ctor, const zend_function_entry * function_list)
 {
 	zend_class_entry ce;
-	
+
 	INIT_CLASS_ENTRY_EX(ce, class_name, strlen(class_name), function_list);
-	*ppce = zend_register_internal_class(&ce TSRMLS_CC);
+	*ppce = zend_register_internal_class(&ce);
 
 	/* entries changed by initialize */
 	if (obj_ctor) {
@@ -53,12 +51,12 @@ PHPAPI void spl_register_std_class(zend_class_entry ** ppce, char * class_name,
 /* }}} */
 
 /* {{{ spl_register_sub_class */
-PHPAPI void spl_register_sub_class(zend_class_entry ** ppce, zend_class_entry * parent_ce, char * class_name, void *obj_ctor, const zend_function_entry * function_list TSRMLS_DC)
+PHPAPI void spl_register_sub_class(zend_class_entry ** ppce, zend_class_entry * parent_ce, char * class_name, void *obj_ctor, const zend_function_entry * function_list)
 {
 	zend_class_entry ce;
-	
+
 	INIT_CLASS_ENTRY_EX(ce, class_name, strlen(class_name), function_list);
-	*ppce = zend_register_internal_class_ex(&ce, parent_ce, NULL TSRMLS_CC);
+	*ppce = zend_register_internal_class_ex(&ce, parent_ce);
 
 	/* entries changed by initialize */
 	if (obj_ctor) {
@@ -70,76 +68,71 @@ PHPAPI void spl_register_sub_class(zend_class_entry ** ppce, zend_class_entry *
 /* }}} */
 
 /* {{{ spl_register_property */
-void spl_register_property( zend_class_entry * class_entry, char *prop_name, int prop_name_len, int prop_flags TSRMLS_DC)
+void spl_register_property( zend_class_entry * class_entry, char *prop_name, int prop_name_len, int prop_flags)
 {
-	zend_declare_property_null(class_entry, prop_name, prop_name_len, prop_flags TSRMLS_CC);
+	zend_declare_property_null(class_entry, prop_name, prop_name_len, prop_flags);
 }
 /* }}} */
 
 /* {{{ spl_add_class_name */
-void spl_add_class_name(zval *list, zend_class_entry * pce, int allow, int ce_flags TSRMLS_DC)
+void spl_add_class_name(zval *list, zend_class_entry *pce, int allow, int ce_flags)
 {
 	if (!allow || (allow > 0 && pce->ce_flags & ce_flags) || (allow < 0 && !(pce->ce_flags & ce_flags))) {
-		size_t len = pce->name_length;
 		zval *tmp;
 
-		if (zend_hash_find(Z_ARRVAL_P(list), pce->name, len+1, (void*)&tmp) == FAILURE) {
-			MAKE_STD_ZVAL(tmp);
-			ZVAL_STRINGL(tmp, pce->name, pce->name_length, 1);
-			zend_hash_add(Z_ARRVAL_P(list), pce->name, len+1, &tmp, sizeof(zval *), NULL);
+		if ((tmp = zend_hash_find(Z_ARRVAL_P(list), pce->name)) == NULL) {
+			zval t;
+			ZVAL_STR_COPY(&t, pce->name);
+			zend_hash_add(Z_ARRVAL_P(list), pce->name, &t);
 		}
 	}
 }
 /* }}} */
 
 /* {{{ spl_add_interfaces */
-void spl_add_interfaces(zval *list, zend_class_entry * pce, int allow, int ce_flags TSRMLS_DC)
+void spl_add_interfaces(zval *list, zend_class_entry * pce, int allow, int ce_flags)
 {
-	zend_uint num_interfaces;
+	uint32_t num_interfaces;
 
 	for (num_interfaces = 0; num_interfaces < pce->num_interfaces; num_interfaces++) {
-		spl_add_class_name(list, pce->interfaces[num_interfaces], allow, ce_flags TSRMLS_CC);
+		spl_add_class_name(list, pce->interfaces[num_interfaces], allow, ce_flags);
 	}
 }
 /* }}} */
 
 /* {{{ spl_add_traits */
-void spl_add_traits(zval *list, zend_class_entry * pce, int allow, int ce_flags TSRMLS_DC)
+void spl_add_traits(zval *list, zend_class_entry * pce, int allow, int ce_flags)
 {
-	zend_uint num_traits;
-  
+	uint32_t num_traits;
+
 	for (num_traits = 0; num_traits < pce->num_traits; num_traits++) {
-		spl_add_class_name(list, pce->traits[num_traits], allow, ce_flags TSRMLS_CC);
+		spl_add_class_name(list, pce->traits[num_traits], allow, ce_flags);
 	}
 }
 /* }}} */
 
 
 /* {{{ spl_add_classes */
-int spl_add_classes(zend_class_entry *pce, zval *list, int sub, int allow, int ce_flags TSRMLS_DC)
+int spl_add_classes(zend_class_entry *pce, zval *list, int sub, int allow, int ce_flags)
 {
 	if (!pce) {
 		return 0;
 	}
-	spl_add_class_name(list, pce, allow, ce_flags TSRMLS_CC);
+	spl_add_class_name(list, pce, allow, ce_flags);
 	if (sub) {
-		spl_add_interfaces(list, pce, allow, ce_flags TSRMLS_CC);
+		spl_add_interfaces(list, pce, allow, ce_flags);
 		while (pce->parent) {
 			pce = pce->parent;
-			spl_add_classes(pce, list, sub, allow, ce_flags TSRMLS_CC);
+			spl_add_classes(pce, list, sub, allow, ce_flags);
 		}
 	}
 	return 0;
 }
 /* }}} */
 
-char * spl_gen_private_prop_name(zend_class_entry *ce, char *prop_name, int prop_len, int *name_len TSRMLS_DC) /* {{{ */
+zend_string * spl_gen_private_prop_name(zend_class_entry *ce, char *prop_name, int prop_len) /* {{{ */
 {
-	char *rv;
-
-	zend_mangle_property_name(&rv, name_len, ce->name, ce->name_length, prop_name, prop_len, 0);
-
-	return rv;
+	return zend_mangle_property_name(ZSTR_VAL(ce->name), ZSTR_LEN(ce->name), prop_name, prop_len, 0);
 }
 /* }}} */
 
