@@ -1,4 +1,3 @@
-	/* $Id: fpm_stdio.c,v 1.22.2.2 2008/12/13 03:32:24 anight Exp $ */
 	/* (c) 2007,2008 Andrei Nigmatulin */
 
 #include "fpm_config.h"
@@ -14,6 +13,7 @@
 
 #include "fpm.h"
 #include "fpm_children.h"
+#include "fpm_cleanup.h"
 #include "fpm_events.h"
 #include "fpm_sockets.h"
 #include "fpm_stdio.h"
@@ -106,6 +106,12 @@ int fpm_stdio_init_child(struct fpm_worker_pool_s *wp) /* {{{ */
 }
 /* }}} */
 
+int fpm_stdio_flush_child() /* {{{ */
+{
+	return write(STDERR_FILENO, "\0", 1);
+}
+/* }}} */
+
 static void fpm_stdio_child_said(struct fpm_event_s *ev, short which, void *arg) /* {{{ */
 {
 	static const int max_buf_size = 1024;
@@ -115,14 +121,16 @@ static void fpm_stdio_child_said(struct fpm_event_s *ev, short which, void *arg)
 	int is_stdout;
 	struct fpm_event_s *event;
 	int fifo_in = 1, fifo_out = 1;
-	int is_last_message = 0;
 	int in_buf = 0;
+	int read_fail = 0, finish_log_stream = 0, create_log_stream;
 	int res;
+	struct zlog_stream *log_stream;
 
 	if (!arg) {
 		return;
 	}
 	child = (struct fpm_child_s *)arg;
+
 	is_stdout = (fd == child->fd_stdout);
 	if (is_stdout) {
 		event = &child->ev_stdout;
@@ -130,32 +138,46 @@ static void fpm_stdio_child_said(struct fpm_event_s *ev, short which, void *arg)
 		event = &child->ev_stderr;
 	}
 
+	create_log_stream = !child->log_stream;
+	if (create_log_stream) {
+		log_stream = child->log_stream = malloc(sizeof(struct zlog_stream));
+		zlog_stream_init_ex(log_stream, ZLOG_WARNING, STDERR_FILENO);
+		zlog_stream_set_decorating(log_stream, child->wp->config->decorate_workers_output);
+		zlog_stream_set_wrapping(log_stream, ZLOG_TRUE);
+		zlog_stream_set_msg_prefix(log_stream, STREAM_SET_MSG_PREFIX_FMT,
+				child->wp->config->name, (int) child->pid, is_stdout ? "stdout" : "stderr");
+		zlog_stream_set_msg_quoting(log_stream, ZLOG_TRUE);
+		zlog_stream_set_is_stdout(log_stream, is_stdout);
+		zlog_stream_set_child_pid(log_stream, (int)child->pid);
+	} else {
+		log_stream = child->log_stream;
+		// if fd type (stdout/stderr) or child's pid is changed,
+		// then the stream will be finished and msg's prefix will be reinitialized
+		if (log_stream->is_stdout != (unsigned int)is_stdout || log_stream->child_pid != (int)child->pid) {
+			zlog_stream_finish(log_stream);
+			zlog_stream_set_msg_prefix(log_stream, STREAM_SET_MSG_PREFIX_FMT,
+					child->wp->config->name, (int) child->pid, is_stdout ? "stdout" : "stderr");
+			zlog_stream_set_is_stdout(log_stream, is_stdout);
+			zlog_stream_set_child_pid(log_stream, (int)child->pid);
+		}
+	}
+
 	while (fifo_in || fifo_out) {
 		if (fifo_in) {
 			res = read(fd, buf + in_buf, max_buf_size - 1 - in_buf);
 			if (res <= 0) { /* no data */
 				fifo_in = 0;
-				if (res < 0 && (errno == EAGAIN || errno == EWOULDBLOCK)) {
-					/* just no more data ready */
-				} else { /* error or pipe is closed */
-
-					if (res < 0) { /* error */
-						zlog(ZLOG_SYSERROR, "unable to read what child say");
-					}
-
-					fpm_event_del(event);
-					is_last_message = 1;
-
-					if (is_stdout) {
-						close(child->fd_stdout);
-						child->fd_stdout = -1;
-					} else {
-						close(child->fd_stderr);
-						child->fd_stderr = -1;
-					}
+				if (res == 0 || (errno != EAGAIN && errno != EWOULDBLOCK)) {
+					/* pipe is closed or error */
+					read_fail = (res < 0) ? res : 1;
 				}
 			} else {
 				in_buf += res;
+				/* if buffer ends with \0, then the stream will be finished */
+				if (!buf[in_buf - 1]) {
+					finish_log_stream = 1;
+					in_buf--;
+				}
 			}
 		}
 
@@ -164,42 +186,48 @@ static void fpm_stdio_child_said(struct fpm_event_s *ev, short which, void *arg)
 				fifo_out = 0;
 			} else {
 				char *nl;
-				int should_print = 0;
-				buf[in_buf] = '\0';
 
-				/* FIXME: there might be binary data */
-
-				/* we should print if no more space in the buffer */
-				if (in_buf == max_buf_size - 1) {
-					should_print = 1;
-				}
-
-				/* we should print if no more data to come */
-				if (!fifo_in) {
-					should_print = 1;
-				}
-
-				nl = strchr(buf, '\n');
-				if (nl || should_print) {
-
-					if (nl) {
-						*nl = '\0';
-					}
-
-					zlog(ZLOG_WARNING, "[pool %s] child %d said into %s: \"%s\"%s", child->wp->config->name,
-					  (int) child->pid, is_stdout ? "stdout" : "stderr", buf, is_last_message ? ", pipe is closed" : "");
-
-					if (nl) {
-						int out_buf = 1 + nl - buf;
-						memmove(buf, buf + out_buf, in_buf - out_buf);
-						in_buf -= out_buf;
-					} else {
-						in_buf = 0;
-					}
+				nl = memchr(buf, '\n', in_buf);
+				if (nl) {
+					/* we should print each new line int the new message */
+					int out_len = nl - buf;
+					zlog_stream_str(log_stream, buf, out_len);
+					zlog_stream_finish(log_stream);
+					/* skip new line */
+					out_len++;
+					/* move data in the buffer */
+					memmove(buf, buf + out_len, in_buf - out_len);
+					in_buf -= out_len;
+				} else if (in_buf == max_buf_size - 1 || !fifo_in) {
+					/* we should print if no more space in the buffer or no more data to come */
+					zlog_stream_str(log_stream, buf, in_buf);
+					in_buf = 0;
 				}
 			}
 		}
 	}
+
+	if (read_fail) {
+		if (create_log_stream) {
+			zlog_stream_set_msg_suffix(log_stream, NULL, ", pipe is closed");
+			zlog_stream_finish(log_stream);
+		}
+		if (read_fail < 0) {
+			zlog(ZLOG_SYSERROR, "unable to read what child say");
+		}
+
+		fpm_event_del(event);
+
+		if (is_stdout) {
+			close(child->fd_stdout);
+			child->fd_stdout = -1;
+		} else {
+			close(child->fd_stderr);
+			child->fd_stderr = -1;
+		}
+	} else if (finish_log_stream) {
+		zlog_stream_finish(log_stream);
+	}
 }
 /* }}} */
 
@@ -289,7 +317,7 @@ int fpm_stdio_open_error_log(int reopen) /* {{{ */
 
 #ifdef HAVE_SYSLOG_H
 	if (!strcasecmp(fpm_global_config.error_log, "syslog")) {
-		openlog(fpm_global_config.syslog_ident, LOG_PID | LOG_CONS, fpm_global_config.syslog_facility);
+		php_openlog(fpm_global_config.syslog_ident, LOG_PID | LOG_CONS, fpm_global_config.syslog_facility);
 		fpm_globals.error_log_fd = ZLOG_SYSLOG;
 		if (fpm_use_error_log()) {
 			zlog_set_fd(fpm_globals.error_log_fd);
